-- YGREC8/YGREC8_def.vhdl
-- created lun. nov. 13 00:47:55 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version lun. nov. 13 02:14:43 CET 2017
-- version ven. nov. 17 02:51:23 CET 2017 : polishing the conditions
-- version dim. nov. 19 08:00:34 CET 2017 : conditions corrected
-- version dim. janv. 28 03:13:47 CET 2018 : LDCx, OVL...
-- Released under the GNU AGPLv3 license
--
-- These are the definitions of YGREC8,
-- shared by the core and other files that instatiate it.
-- Borrowed from the microYASEP/yasep_def.vhdl

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;

package ygrec8_def is
  subtype SL     is std_logic;
  subtype SLV2   is std_logic_vector( 1 downto 0);
  subtype SLV3   is std_logic_vector( 2 downto 0);
  subtype SLV4   is std_logic_vector( 3 downto 0);
  subtype SLV5   is std_logic_vector( 4 downto 0);
  subtype SLV6   is std_logic_vector( 5 downto 0);
  subtype SLV8   is std_logic_vector( 7 downto 0);
  subtype SLV9   is std_logic_vector( 8 downto 0);
  subtype SLV10  is std_logic_vector( 9 downto 0);
  subtype SLV16  is std_logic_vector(15 downto 0);
  subtype SLV17  is std_logic_vector(16 downto 0);

  type opcode_array_type  is array (0 to 15) of string(4 downto 1);
  constant Opcode_str_array : opcode_array_type := (
    "OR  ", "XOR ", "AND ", "ANDN",
    "CMPU", "CMPS", "SUB ", "ADD ",
    "SHR ", "SHL ", "SAR ", "ROL ",
    "IN  ", "LDCL", "MOV ", "CALL" );

  constant Op_OR   : SLV4 := "0000";
  constant Op_XOR  : SLV4 := "0001";
  constant Op_AND  : SLV4 := "0010";
  constant Op_ANDN : SLV4 := "0011";
  constant Op_CMPU : SLV4 := "0100";
  constant Op_CMPS : SLV4 := "0101";
  constant Op_SUB  : SLV4 := "0110";
  constant Op_ADD  : SLV4 := "0111";
  constant Op_SHR  : SLV4 := "1000";
  constant Op_SHL  : SLV4 := "1001";
  constant Op_SAR  : SLV4 := "1010";
  constant Op_ROL  : SLV4 := "1011";
  constant Op_IN   : SLV4 := "1100";
  constant Op_LDCL : SLV4 := "1101";
  constant Op_MOV  : SLV4 := "1110";
  constant Op_CALL : SLV4 := "1111";

  type string_array_type_7 is array (0 to 7) of string(2 downto 1);
  constant Reg_str_array : string_array_type_7 := (
    "D1", "A1", "D2", "A2", "R1", "R2", "R3", "PC" );

  constant Reg_D1 : SLV3 := "000";
  constant Reg_A1 : SLV3 := "001";
  constant Reg_D2 : SLV3 := "010";
  constant Reg_A2 : SLV3 := "011";
  constant Reg_R1 : SLV3 := "100";
  constant Reg_R2 : SLV3 := "101";
  constant Reg_R3 : SLV3 := "110";
  constant Reg_PC : SLV3 := "111";

  type string_array_type_15 is array (0 to 15) of string(4 downto 1);
  constant Cond_str_array : string_array_type_15 := (
    "ALWS", "IFNC", "IFN ", "IFNZ", "IF0 ", "IF1 ", "IF2 ", "IF3 ",
    "NEVR", "IFC ", "IFP ", "IFZ ", "IFN0", "IFN1", "IFN2", "IFN3");
  constant Cond_A  : SLV4 := "0000";
  constant Cond_C  : SLV4 := "0001";
  constant Cond_S  : SLV4 := "0010";
  constant Cond_NZ : SLV4 := "0011";
  constant Cond_0  : SLV4 := "0100";
  constant Cond_1  : SLV4 := "0101";
  constant Cond_2  : SLV4 := "0110";
  constant Cond_3  : SLV4 := "0111";

  constant Cond_N  : SLV4 := "1000";
  constant Cond_NC : SLV4 := "1001";
  constant Cond_NS : SLV4 := "1010";
  constant Cond_Z  : SLV4 := "1011";
  constant Cond_N0 : SLV4 := "1100";
  constant Cond_N1 : SLV4 := "1101";
  constant Cond_N2 : SLV4 := "1110";
  constant Cond_N3 : SLV4 := "1111";

end ygrec8_def;

--package body ygrec8_def is
-- empty
--end ygrec8_def;
