-- YGREC8/YGREC8_utils.vhdl
-- created lun. nov. 13 01:02:38 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version dim. nov. 19 03:49:07 CET 2017 : added Num_to_int
-- Released under the GNU AGPLv3 license
--
-- These are some useful functions for the YGREC8,
-- some functions are shamelessly "borrowed" from yasep_utils.vhdl

Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;

package ygrec8_utils is
  function safe_to_integer(u:std_logic_vector) return integer;
  function SLV_to_bin(s:std_logic_vector) return string;
  function SLV4_to_Hex(s:std_logic_vector(3 downto 0)) return character;
  function SLV_to_Hex(s:std_logic_vector) return string;

  function Num_to_int(txt, context: string) return integer;
  -- useful for txt to int conversion:
  type char_base_type is array (character) of integer;
  constant base_lookup : char_base_type := (
   '0'=>0,  '1'=>1,  '2'=>2,  '3'=>3,  '4'=>4,
   '5'=>5,  '6'=>6,  '7'=>7,  '8'=>8,  '9'=>9,
   'A'=>10, 'B'=>11, 'C'=>12, 'D'=>13, 'E'=>14,
   'F'=>15, others=>-1);

end ygrec8_utils;

package body ygrec8_utils is

  function safe_to_integer(u:std_logic_vector) return integer is
  begin
   if Is_X(u) then
      return 0;
    else
      return to_integer(unsigned(u));
    end if;
  end safe_to_integer;

  function SLV_to_bin(s:std_logic_vector) return string is
    variable t : string(s'range);
    variable u : string(3 downto 1);
  begin
    for i in s'range loop
      u := std_logic'image(s(i));
      t(i) := u(2);
    end loop;
    return t;
  end SLV_to_bin;

  function SLV4_to_Hex(s: std_logic_vector(3 downto 0)) return character is
  begin
    case s is
      when "0000" => return '0';
      when "0001" => return '1';
      when "0010" => return '2';
      when "0011" => return '3';
      when "0100" => return '4';
      when "0101" => return '5';
      when "0110" => return '6';
      when "0111" => return '7';
      when "1000" => return '8';
      when "1001" => return '9';
      when "1010" => return 'A';
      when "1011" => return 'B';
      when "1100" => return 'C';
      when "1101" => return 'D';
      when "1110" => return 'E';
      when "1111" => return 'F';
      when others => return '?';
    end case;
  end SLV4_to_Hex;

  function SLV_to_Hex(s:std_logic_vector) return string is
    variable u: std_logic_vector(s'length+3 downto 0);
    variable t: string((s'length-1)/4 downto 0);
    variable i: integer:= 0;
    variable j: integer:= 0;
  begin
    u:= "0000" & s;
    while (i < s'length) loop
      t(j) := SLV4_to_Hex(u(i+3 downto i));
      i := i+4;
      j := j+1;
    end loop;
    return t;
  end SLV_to_Hex;

  function Num_to_int(txt, context: string) return integer
  is
    variable c : character;
    variable d, i, j, n, n2, high : integer := 0;
    variable base : integer := 10;
    variable sign : integer := 1;
  begin
    -- txt should contain only "valid characters" and no space.
    -- overflow is not well managed though. At least is must work
    -- nicely up to 4K.

    -- but first, read the base at the trailing end of the string
    high := txt'high; -- and change the length accordingly.
    assert high >= 1 report context & " invalid number, empty string" severity failure;
    case txt(high) is
      when 'H' => base := 16 ; high := high-1;
      when 'B' => base := 2 ; high := high-1;
      when others =>
    end case;

    -- get the sign
    i := 1;
    if high > 1 then
      c := txt(i);
      if c='-' then
        sign := -1;
        i := i+1;
      end if;
    end if;

    convert: while (i <= high) loop
      c := txt(i);
      d := base_lookup(c);
      assert d  >= 0  report context & " invalid digit '" & c & "'" severity failure;
      assert d < base report context & " wrong base for digit '" & c & "'" severity failure;
      n2 := (n*base) + d;
      assert n <= n2  report context & " invalid number, overflow" severity failure;
      n := n2;
      i := i+1;
      j := j+1;
    end loop convert;

    assert j > 0 report context & " invalid number, no digit? '" & txt & "'" severity failure;

    return sign*n;
  end Num_to_int;



--file f_config : text is in "/tmp/config_file.txt";
--
--function config_read
--return integer is
--variable rl : line;
--variable res : integer;
--begin
--  readline(f_config,rl);
--  read(rl,res);
--  return res;
--end function config_read;


end ygrec8_utils;
