#!/bin/bash
# file test.sh
# created by Yann Guidon / ygdes.com
# version sam. nov. 11 19:26:36 CET 2017
# version lun. nov. 13 02:34:03 CET 2017
# version jeu. nov. 16 03:07:00 CET 2017
# version mer. oct. 17 03:38:23 CEST 2018 : INC8_tiles.vhdl & INC8_gates.vhdl
# version mar. nov. 19 06:02:18 CET 2019 : integrated in the proasic3v2 testbench
# version jeu. nov. 21 06:57:25 CET 2019 : trace changes

if [ ! -f ../proasic3v2/proasic3-obj93.cf  ]; then
  echo -n '--> going to ' ; pushd ../proasic3v2/
  ./build.sh
  echo -n '<-- back to ' ; popd
fi

VER="--std=93"
PA3="-P../proasic3v2"
VHDLIB="YGREC8_def.vhdl YGREC8_utils.vhdl"

function run_test() {
  # arg.1 : INC8 version/file
  # arg.2 : message
  # optional $PA3
  rm -f inc8 inc8_tb work-obj93.cf *.o
  ghdl -a $3  $VHDLIB  $1 && ghdl -e $3 INC8    && ./inc8    && echo INC8 $2 standalone: OK &&
  ghdl -a $3 INC8_tb.vhdl && ghdl -e $3 INC8_tb && ./inc8_tb && echo INC8 $2 testbench:  OK
}

echo "Testing the reference designs :"
run_test INC8.vhdl        ""        ""     &&
run_test INC8_A3P.vhdl    "A3P"     ""     &&
run_test INC8_gates.vhdl  "gates"   ""     &&
run_test INC8_tiles.vhdl  "tiles"   "$PA3" &&

#echo "OK now let's probe the gates" &&
rm -f inc8 inc8_tb work-obj93.cf *.o &&
ghdl -a $PA3 $VHDLIB INC8_tiles.vhdl INC8_tb2.vhdl && ghdl -e $PA3 INC8_tb && ./inc8_tb &&
# it now uses a modified version of INC8_tb.vhdl, with additional includes and instances.

# extract the gates list :
DEPTH_LIST=$(./inc8_tb -ggate_select_number=-1 |grep '  gate#' |sed 's/.*(trace):lut//' )
k=1  # first gate is n°1
l=0
# scan the list to extract the depth
for i in $DEPTH_LIST ; do
  # for each bit in the LUT:
  for j in $(seq 0 $((i-1)) ) ; do
    echo -n $l " " >&2
    echo 'Run #'$l  'Gate #'$k 'LUT'$i'['$j']'
    (./inc8_tb -gflip=1 -ggate_select_number=$k -gbit_select_number=$j &&
          echo 'failure not found :-(' ) || echo 'Failure detected : great !'
    echo ""
    l=$((l+1))
  done;
  k=$((k+1))
done > failures.log &&

( grep 'failure not found :-(' failures.log &&
    echo "Oh, some failures have not been detected") ||
      ( echo "All the bits have been successfully checked !" &&
rm -f inc8 inc8_tb work-obj93.cf *.o )
